/* --COPYRIGHT--,BSD
 * Copyright (c) 2015, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
#include <stdint.h>

#include <msp430.h>

#include "grlib/Library/grlib.h"

#include "button.h"

/* Graphic library context */
extern tContext g_sContext;

/* Math mode float button. */
tRectangle buttonMathModeFloatBoarder = {
    BUTTONS_START_X,
    BUTTONS_START_Y1,
    BUTTONS_START_X + BUTTON_WIDTH,
    BUTTONS_START_Y1 + BUTTON_HEIGHT
};
tRectangle buttonMathModeFloatFill = {
    BUTTONS_START_X + BUTTON_BOARDER,
    BUTTONS_START_Y1 + BUTTON_BOARDER,
    BUTTONS_START_X + BUTTON_WIDTH - BUTTON_BOARDER,
    BUTTONS_START_Y1 + BUTTON_HEIGHT - BUTTON_BOARDER
};

/* Math mode IQmathLib button. */
tRectangle buttonMathModeIQmathBoarder = {
    BUTTONS_START_X,
    BUTTONS_START_Y2,
    BUTTONS_START_X + BUTTON_WIDTH,
    BUTTONS_START_Y2 + BUTTON_HEIGHT
};
tRectangle buttonMathModeIQmathFill = {
    BUTTONS_START_X + BUTTON_BOARDER,
    BUTTONS_START_Y2 + BUTTON_BOARDER,
    BUTTONS_START_X + BUTTON_WIDTH - BUTTON_BOARDER,
    BUTTONS_START_Y2 + BUTTON_HEIGHT - BUTTON_BOARDER
};

/* Shape cube button. */
tRectangle buttonShapeCubeBoarder = {
    BUTTONS_START_X,
    BUTTONS_START_Y3,
    BUTTONS_START_X + BUTTON_WIDTH,
    BUTTONS_START_Y3 + BUTTON_HEIGHT
};
tRectangle buttonShapeCubeFill = {
    BUTTONS_START_X + BUTTON_BOARDER,
    BUTTONS_START_Y3 + BUTTON_BOARDER,
    BUTTONS_START_X + BUTTON_WIDTH - BUTTON_BOARDER,
    BUTTONS_START_Y3 + BUTTON_HEIGHT - BUTTON_BOARDER
};

/* Shape dodecahedron button. */
tRectangle buttonShapeDodecahedronBoarder = {
    BUTTONS_START_X,
    BUTTONS_START_Y4,
    BUTTONS_START_X + BUTTON_WIDTH,
    BUTTONS_START_Y4 + BUTTON_HEIGHT
};
tRectangle buttonShapeDodecahedronFill = {
    BUTTONS_START_X + BUTTON_BOARDER,
    BUTTONS_START_Y4 + BUTTON_BOARDER,
    BUTTONS_START_X + BUTTON_WIDTH - BUTTON_BOARDER,
    BUTTONS_START_Y4 + BUTTON_HEIGHT - BUTTON_BOARDER
};

/* Shape hypercube button. */
tRectangle buttonShapeHypercubeBoarder = {
    BUTTONS_START_X,
    BUTTONS_START_Y5,
    BUTTONS_START_X + BUTTON_WIDTH,
    BUTTONS_START_Y5 + BUTTON_HEIGHT
};
tRectangle buttonShapeHypercubeFill = {
    BUTTONS_START_X + BUTTON_BOARDER,
    BUTTONS_START_Y5 + BUTTON_BOARDER,
    BUTTONS_START_X + BUTTON_WIDTH - BUTTON_BOARDER,
    BUTTONS_START_Y5 + BUTTON_HEIGHT - BUTTON_BOARDER
};

void button_drawButton(tRectangle *boarder,
                       uint32_t boarderColor,
                       tRectangle *fill,
                       uint32_t fillColor,
                       char *text)
{
    /* Draw the boarder rectangle. */
    GrContextForegroundSet(&g_sContext, boarderColor);
    GrRectFill(&g_sContext, boarder);

    /* Draw the fill rectangle. */
    GrContextForegroundSet(&g_sContext, fillColor);
    GrRectFill(&g_sContext, fill);

    /* Draw the text. */
    GrContextForegroundSet(&g_sContext, ClrWhite);
    GrContextFontSet(&g_sContext, &g_sFontCmss24);
    GrStringDrawCentered(
        &g_sContext,
        (int8_t *)text,
        AUTO_STRING_LENGTH,
        (fill->sXMin + fill->sXMax) / 2,
        (fill->sYMin + fill->sYMax) / 2,
        TRANSPARENT_TEXT
        );

    return;
}
