/* --COPYRIGHT--,BSD
 * Copyright (c) 2015, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
#ifndef DEMO_H_
#define DEMO_H_

#include <stdbool.h>
#include <stdint.h>

#define GLOBAL_Q    8

#include "QmathLib.h"
#include "IQmathLib.h"

/*
 * Type definition for 2D uin16_t vector.
 */
typedef struct ui16Vector2D
{
    uint16_t x, y;
} ui16Vector2D;

/*
 * Type definition for 3D _q vector.
 */
typedef struct qVector3D
{
    _q x, y, z;
} qVector3D;

/* Type definition for 3D float vector.
 */
typedef struct fVector3D
{
    float x, y, z;
} fVector3D;

/* Type definition for 4D _q vector.
 */
typedef struct qVector4D
{
    _q x, y, z, w;
} qVector4D;

/*
 * Type definition for 4D float vector.
 */
typedef struct fVector4D
{
    float x, y, z, w;
} fVector4D;

/*
 * Demo parameters.
 */
#define DEMO_SIZE_MIN       24      // size scaling minimum
#define DEMO_SIZE_MAX       32      // size scaling maximum
#define DEMO_SIZE_SPEED     0.25    // size scaling speed
#define DEMO_COLOR_SPEED    10      // color change speed
#define DEMO_ROTATION_SPEED 0.06    // radians per rotation

/*
 * Math mode definitions.
 */
#define DEMO_MODE_FLOAT         0
#define DEMO_MODE_FIXED         1

/*
 * Shape definitions.
 */
#define DEMO_SHAPE_CUBE         0
#define DEMO_SHAPE_DODECAHEDRON 1
#define DEMO_SHAPE_HYPERCUBE    2

/*
 * Color definitions for rotating colors.
 */
#define DEMO_INDEX_BLUE         0
#define DEMO_INDEX_GREEN        1
#define DEMO_INDEX_RED          2
#define DEMO_DIRECTION_UP       0
#define DEMO_DIRECTION_DOWN     1

/*
 * Display definitions.
 */
#define DEMO_XOFFSET    (LCD_HORIZONTAL_MAX / 4 - 1)
#define DEMO_YOFFSET    (LCD_VERTICAL_MAX / 2 - 8)

/*
 * Globals
 */
extern uint16_t g_ui16MathMode;         // math mode
extern uint16_t g_ui16Shape;            // shape
extern uint32_t g_i32GraphicsCycles;    // Cycle count for graphics calculation
extern uint32_t g_i32DisplayCycles;     // Cycle count for display update

/*
 * Function Prototypes
 */
extern void demo_updateFrame(uint16_t mode,
                             uint16_t shape);
extern void demo_ui16Draw2D(ui16Vector2D shape[][2],
                            uint16_t length);
extern void demo_fConvert3Dto2D(fVector3D shape3D[][2],
                                ui16Vector2D shape2D[][2],
                                uint16_t length,
                                bool scale);
extern void demo_fRotate3D(const fVector3D shape[][2],
                           fVector3D rotated[][2],
                           uint16_t length);
extern void demo_qConvert3Dto2D(qVector3D shape3D[][2],
                                ui16Vector2D shape2D[][2],
                                uint16_t length,
                                bool scale);
extern void demo_qRotate3D(const qVector3D shape[][2],
                           qVector3D rotated[][2],
                           uint16_t length);
extern void demo_fRotate4Dto3D(const fVector4D shape[][2],
                               fVector3D rotated[][2],
                               uint16_t length);
extern void demo_qRotate4Dto3D(const qVector4D shape[][2],
                               qVector3D rotated[][2],
                               uint16_t length);
extern int32_t demo_getNextColor(void);

#endif  //DEMO_H_
