/* --COPYRIGHT--,BSD
 * Copyright (c) 2015, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//*********************************************************************************
//
// Copyright (c) 2008-2014 Texas Instruments Incorporated.  All rights reserved.
// Software License Agreement
//
// Texas Instruments (TI) is supplying this software for use solely and
// exclusively on TI's microcontroller products. The software is owned by
// TI and/or its suppliers, and is protected under applicable copyright
// laws. You may not combine this software with "viral" open-source
// software in order to form a larger program.
//
// THIS SOFTWARE IS PROVIDED "AS IS" AND WITH ALL FAULTS.
// NO WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT
// NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. TI SHALL NOT, UNDER ANY
// CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
// DAMAGES, FOR ANY REASON WHATSOEVER.
//
//*****************************************************************************

#ifndef __HAL_SLAA548_KITRONIX320X240X16_SSD2119_16BIT_H__
#define __HAL_SLAA548_KITRONIX320X240X16_SSD2119_16BIT_H__

//*****************************************************************************
//
// Include Files
//
//*****************************************************************************

//*****************************************************************************
//
// Defines for the pins that are used to communicate with the SSD2119.
//
//*****************************************************************************
#if defined(__MSP430F5529__)
//LCD_SDC Pin
#define LCD_SDC_DIR             P2DIR
#define LCD_SDC_OUT             P2OUT
#define LCD_SDC_PIN             BIT7

//LCD_SCS Pin
#define LCD_SCS_DIR             P2DIR
#define LCD_SCS_OUT             P2OUT
#define LCD_SCS_PIN             BIT6

//LCD_RST Pin
#define LCD_RST_DIR             P3DIR
#define LCD_RST_OUT             P3OUT
#define LCD_RST_PIN             BIT7

//LCD_PWM Pin (backlight)
#define LCD_PWM_DIR             P2DIR
#define LCD_PWM_OUT             P2OUT
#define LCD_PWM_PIN             BIT5

static inline void HAL_configureSpi(void)
{
    //LCD_SDI Pin
    P3SEL |= BIT0;

    //LCD_SCL Pin
    P3SEL |= BIT2;

    //Configure UCB0 for SPI
    UCB0CTL1 |= UCSWRST;
    UCB0CTL0 |= UCMST + UCSYNC + UCCKPL + UCMSB;
    UCB0CTL1 |= UCSSEL_2;
    UCB0BR0 = 0x00;
    UCB0BR1 = 0;
    UCB0CTL1 &= ~UCSWRST;
}

#elif defined(__MSP430FR5969__)
//LCD_SDC Pin
#define LCD_SDC_DIR             P3DIR
#define LCD_SDC_OUT             P3OUT
#define LCD_SDC_PIN             BIT4

//LCD_SCS Pin
#define LCD_SCS_DIR             P1DIR
#define LCD_SCS_OUT             P1OUT
#define LCD_SCS_PIN             BIT5

//LCD_RST Pin
#define LCD_RST_DIR             P3DIR
#define LCD_RST_OUT             P3OUT
#define LCD_RST_PIN             BIT0

//LCD_PWM Pin (backlight)
#define LCD_PWM_DIR             P1DIR
#define LCD_PWM_OUT             P1OUT
#define LCD_PWM_PIN             BIT2

static inline void HAL_configureSpi(void)
{
    //LCD_SDI Pin
    P1SEL0 &= ~BIT6;
    P1SEL1 |= BIT6;

    //LCD_SCL Pin
    P2SEL0 &= ~BIT2;
    P2SEL1 |= BIT2;

    //Configure UCB0 for SPI
    UCB0CTLW0 = UCSWRST;
    UCB0CTLW0 |= UCMST + UCSYNC + UCCKPL + UCMSB;
    UCB0CTLW0 |= UCSSEL_2;
    UCB0BR0 = 0x00;
    UCB0BR1 = 0x00;
    UCA0MCTLW = 0x00;
    UCB0CTLW0 &= ~UCSWRST;
}

#else
#error "Device not defined in HAL_kitronix320x240x16_ssd2119_spi.h"
#endif

//*****************************************************************************
//
// Select the LCD for writing using the SCS pin.
//
//*****************************************************************************
static inline void SelectLCD(void)
{
    // Wait for any SPI transmission to complete before setting the LCD_SCS signal.
    while(UCB0STAT & UCBUSY)
    {
        ;
    }
    LCD_SCS_OUT &= ~LCD_SCS_PIN;
}

//*****************************************************************************
//
// Deselect the LCD for writing using the SCS pin.
//
//*****************************************************************************
static inline void DeselectLCD(void)
{
    // Wait for any SPI transmission to complete before setting the LCD_SCS signal.
    while(UCB0STAT & UCBUSY)
    {
        ;
    }
    LCD_SCS_OUT |= LCD_SCS_PIN;
}

//*****************************************************************************
//
// Writes a data word to the SSD2119.  This function implements the basic SPI
// interface to the LCD display.
//
//*****************************************************************************
static inline void WriteData(uint16_t Data)
{
    uint8_t ui8Data;

    // Calculate the high byte, wait for buffer to become empty and transmit.
    ui8Data = (uint8_t)(Data >> 8);
    while(!(UCB0IFG & UCTXIFG))
    {
        ;
    }
    UCB0TXBUF = ui8Data;

    // Calculate the low byte, wait for buffer to become empty and transmit.
    ui8Data = (uint8_t)(Data & 0xff);
    while(!(UCB0IFG & UCTXIFG))
    {
        ;
    }
    UCB0TXBUF = ui8Data;
}

//*****************************************************************************
//
// Writes a command to the SSD2119.  This function implements the basic SPI
// interface to the LCD display.
//
//*****************************************************************************
static inline void WriteCommand(uint8_t Data)
{
    // Wait for any SPI transmission to complete before setting the LCD_SDC signal.
    while(UCB0STAT & UCBUSY)
    {
        ;
    }
    LCD_SDC_OUT &= ~LCD_SDC_PIN;

    // Transmit the command.
    UCB0TXBUF = Data;
    while(UCB0STAT & UCBUSY)
    {
        ;
    }

    // Set the LCD_SDC signal high, indicating that following writes are data.
    LCD_SDC_OUT |= LCD_SDC_PIN;
}

//*****************************************************************************
//
// Prototypes.
//
//*****************************************************************************
extern void InitLCDInterfaceSPI(void);

#endif //__HAL_SLAA548_KITRONIX320X240X16_SSD2119_16BIT_H__
