/* --COPYRIGHT--,BSD
 * Copyright (c) 2015, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
#include "demo.h"
#include "shapes.h"

/*
 * Cube _q coordinate points
 */
const qVector3D gc_qCube3D[SHAPE_CUBE_SIZE][2] = {
    {
        {-_Q(ONE), -_Q(ONE), -_Q(ONE)},     { _Q(ONE), -_Q(ONE), -_Q(ONE)}
    },                                                                          // 0
    {
        {-_Q(ONE), -_Q(ONE), -_Q(ONE)},     {-_Q(ONE),  _Q(ONE), -_Q(ONE)}
    },                                                                          // 1
    {
        {-_Q(ONE),  _Q(ONE), -_Q(ONE)},     { _Q(ONE),  _Q(ONE), -_Q(ONE)}
    },                                                                          // 2
    {
        { _Q(ONE), -_Q(ONE), -_Q(ONE)},     { _Q(ONE),  _Q(ONE), -_Q(ONE)}
    },                                                                          // 3
    {
        {-_Q(ONE), -_Q(ONE),  _Q(ONE)},     { _Q(ONE), -_Q(ONE),  _Q(ONE)}
    },                                                                          // 4
    {
        {-_Q(ONE), -_Q(ONE),  _Q(ONE)},     {-_Q(ONE),  _Q(ONE),  _Q(ONE)}
    },                                                                          // 5
    {
        {-_Q(ONE),  _Q(ONE),  _Q(ONE)},     { _Q(ONE),  _Q(ONE),  _Q(ONE)}
    },                                                                          // 6
    {
        { _Q(ONE), -_Q(ONE),  _Q(ONE)},     { _Q(ONE),  _Q(ONE),  _Q(ONE)}
    },                                                                          // 7
    {
        {-_Q(ONE), -_Q(ONE),  _Q(ONE)},     {-_Q(ONE), -_Q(ONE), -_Q(ONE)}
    },                                                                          // 8
    {
        {-_Q(ONE),  _Q(ONE),  _Q(ONE)},     {-_Q(ONE),  _Q(ONE), -_Q(ONE)}
    },                                                                          // 9
    {
        { _Q(ONE),  _Q(ONE),  _Q(ONE)},     { _Q(ONE),  _Q(ONE), -_Q(ONE)}
    },                                                                          // 10
    {
        { _Q(ONE), -_Q(ONE),  _Q(ONE)},     { _Q(ONE), -_Q(ONE), -_Q(ONE)}
    }                                                                           // 11
};

/*
 * Cube float coordinate points
 */
const fVector3D gc_fCube3D[SHAPE_CUBE_SIZE][2] = {
    {
        {-ONE, -ONE, -ONE},     { ONE, -ONE, -ONE}
    },                                              // 0
    {
        {-ONE, -ONE, -ONE},     {-ONE,  ONE, -ONE}
    },                                              // 1
    {
        {-ONE,  ONE, -ONE},     { ONE,  ONE, -ONE}
    },                                              // 2
    {
        { ONE, -ONE, -ONE},     { ONE,  ONE, -ONE}
    },                                              // 3
    {
        {-ONE, -ONE,  ONE},     { ONE, -ONE,  ONE}
    },                                              // 4
    {
        {-ONE, -ONE,  ONE},     {-ONE,  ONE,  ONE}
    },                                              // 5
    {
        {-ONE,  ONE,  ONE},     { ONE,  ONE,  ONE}
    },                                              // 6
    {
        { ONE, -ONE,  ONE},     { ONE,  ONE,  ONE}
    },                                              // 7
    {
        {-ONE, -ONE,  ONE},     {-ONE, -ONE, -ONE}
    },                                              // 8
    {
        {-ONE,  ONE,  ONE},     {-ONE,  ONE, -ONE}
    },                                              // 9
    {
        { ONE,  ONE,  ONE},     { ONE,  ONE, -ONE}
    },                                              // 10
    {
        { ONE, -ONE,  ONE},     { ONE, -ONE, -ONE}
    }                                               // 11
};

/*
 * Dodecahedron _q coordinate points
 */
const qVector3D gc_qDodecahedron3D[SHAPE_DODECAHEDRON_SIZE][2] = {
    {
        {_Q(-INV_PHI), _Q(PHI), _Q(ZERO)},     {_Q(INV_PHI), _Q(PHI), _Q(ZERO)}
    },                                                                              //  0
    {
        {_Q(-INV_PHI), _Q(PHI), _Q(ZERO)},     {_Q(-ONE), _Q(ONE), _Q(ONE)}
    },                                                                              //  1
    {
        {_Q(-INV_PHI), _Q(PHI), _Q(ZERO)},     {_Q(-ONE), _Q(ONE), _Q(-ONE)}
    },                                                                              //  2
    {
        {_Q(ONE), _Q(ONE), _Q(ONE)},           {_Q(INV_PHI), _Q(PHI), _Q(ZERO)}
    },                                                                              //  3
    {
        {_Q(ONE), _Q(ONE), _Q(-ONE)},          {_Q(INV_PHI), _Q(PHI), _Q(ZERO)}
    },                                                                              //  4
    {
        {_Q(-INV_PHI), _Q(-PHI),
         _Q(ZERO)},     {_Q(INV_PHI), _Q(-PHI), _Q(ZERO)}
    },                                                                              //  5
    {
        {_Q(-INV_PHI), _Q(-PHI), _Q(ZERO)},     {_Q(-ONE), _Q(-ONE), _Q(ONE)}
    },                                                                              //  6
    {
        {_Q(-INV_PHI), _Q(-PHI), _Q(ZERO)},     {_Q(-ONE), _Q(-ONE), _Q(-ONE)}
    },                                                                              //  7
    {
        {_Q(ONE), _Q(-ONE),
         _Q(ONE)},           {_Q(INV_PHI), _Q(-PHI), _Q(ZERO)}
    },                                                                              //  8
    {
        {_Q(ONE), _Q(-ONE),
         _Q(-ONE)},          {_Q(INV_PHI), _Q(-PHI), _Q(ZERO)}
    },                                                                              //  9
    {
        {_Q(ZERO), _Q(INV_PHI), _Q(PHI)},         {_Q(ZERO), _Q(-INV_PHI), _Q(
                                                       PHI)}
    },                                                                              //  10
    {
        {_Q(ZERO), _Q(INV_PHI), _Q(PHI)},         {_Q(-ONE), _Q(ONE), _Q(ONE)}
    },                                                                              //  11
    {
        {_Q(ZERO), _Q(INV_PHI), _Q(PHI)},         {_Q(ONE), _Q(ONE), _Q(ONE)}
    },                                                                              //  12
    {
        {_Q(-ONE), _Q(-ONE), _Q(ONE)},            {_Q(ZERO), _Q(-INV_PHI), _Q(
                                                       PHI)}
    },                                                                              //  13
    {
        {_Q(ONE), _Q(-ONE), _Q(ONE)},             {_Q(ZERO), _Q(-INV_PHI), _Q(
                                                       PHI)}
    },                                                                              //  14
    {
        {_Q(ZERO), _Q(INV_PHI),
         _Q(-PHI)},     {_Q(ZERO), _Q(-INV_PHI), _Q(-PHI)}
    },                                                                              //  15
    {
        {_Q(ZERO), _Q(INV_PHI), _Q(-PHI)},     {_Q(-ONE), _Q(ONE), _Q(-ONE)}
    },                                                                              //  16
    {
        {_Q(ZERO), _Q(INV_PHI), _Q(-PHI)},     {_Q(ONE), _Q(ONE), _Q(-ONE)}
    },                                                                              //  17
    {
        {_Q(-ONE), _Q(-ONE),
         _Q(-ONE)},        {_Q(ZERO), _Q(-INV_PHI), _Q(-PHI)}
    },                                                                              //  18
    {
        {_Q(ONE), _Q(-ONE),
         _Q(-ONE)},         {_Q(ZERO), _Q(-INV_PHI), _Q(-PHI)}
    },                                                                              //  19
    {
        {_Q(PHI), _Q(ZERO), _Q(INV_PHI)},         {_Q(PHI), _Q(ZERO), _Q(
                                                       -INV_PHI)}
    },                                                                              //  20
    {
        {_Q(PHI), _Q(ZERO), _Q(INV_PHI)},         {_Q(ONE), _Q(ONE), _Q(ONE)}
    },                                                                              //  21
    {
        {_Q(PHI), _Q(ZERO), _Q(INV_PHI)},         {_Q(ONE), _Q(-ONE), _Q(ONE)}
    },                                                                              //  22
    {
        {_Q(ONE), _Q(ONE),
         _Q(-ONE)},            {_Q(PHI), _Q(ZERO), _Q(-INV_PHI)}
    },                                                                              //  23
    {
        {_Q(ONE), _Q(-ONE), _Q(-ONE)},            {_Q(PHI), _Q(ZERO), _Q(
                                                       -INV_PHI)}
    },                                                                              //  24
    {
        {_Q(-PHI), _Q(ZERO),
         _Q(INV_PHI)},     {_Q(-PHI), _Q(ZERO), _Q(-INV_PHI)}
    },                                                                              //  25
    {
        {_Q(-PHI), _Q(ZERO), _Q(INV_PHI)},     {_Q(-ONE), _Q(ONE), _Q(ONE)}
    },                                                                              //  26
    {
        {_Q(-PHI), _Q(ZERO), _Q(INV_PHI)},     {_Q(-ONE), _Q(-ONE), _Q(ONE)}
    },                                                                              //  27
    {
        {_Q(-ONE), _Q(ONE),
         _Q(-ONE)},         {_Q(-PHI), _Q(ZERO), _Q(-INV_PHI)}
    },                                                                              //  28
    {
        {_Q(-ONE), _Q(-ONE),
         _Q(-ONE)},        {_Q(-PHI), _Q(ZERO), _Q(-INV_PHI)}
    },                                                                              //  29
};

/*
 * Dodecahedron float coordinate points
 */
const fVector3D gc_fDodecahedron3D[SHAPE_DODECAHEDRON_SIZE][2] = {
    {
        {-INV_PHI, PHI, ZERO},     {INV_PHI, PHI, ZERO}
    },                                                      //  0
    {
        {-INV_PHI, PHI, ZERO},     {-ONE, ONE, ONE}
    },                                                      //  1
    {
        {-INV_PHI, PHI, ZERO},     {-ONE, ONE, -ONE}
    },                                                      //  2
    {
        {ONE, ONE, ONE},           {INV_PHI, PHI, ZERO}
    },                                                      //  3
    {
        {ONE, ONE, -ONE},          {INV_PHI, PHI, ZERO}
    },                                                      //  4
    {
        {-INV_PHI, -PHI, ZERO},     {INV_PHI, -PHI, ZERO}
    },                                                      //  5
    {
        {-INV_PHI, -PHI, ZERO},     {-ONE, -ONE, ONE}
    },                                                      //  6
    {
        {-INV_PHI, -PHI, ZERO},     {-ONE, -ONE, -ONE}
    },                                                      //  7
    {
        {ONE, -ONE, ONE},           {INV_PHI, -PHI, ZERO}
    },                                                      //  8
    {
        {ONE, -ONE, -ONE},          {INV_PHI, -PHI, ZERO}
    },                                                      //  9
    {
        {ZERO, INV_PHI, PHI},         {ZERO, -INV_PHI, PHI}
    },                                                      //  10
    {
        {ZERO, INV_PHI, PHI},         {-ONE, ONE, ONE}
    },                                                      //  11
    {
        {ZERO, INV_PHI, PHI},         {ONE, ONE, ONE}
    },                                                      //  12
    {
        {-ONE, -ONE, ONE},            {ZERO, -INV_PHI, PHI}
    },                                                      //  13
    {
        {ONE, -ONE, ONE},             {ZERO, -INV_PHI, PHI}
    },                                                      //  14
    {
        {ZERO, INV_PHI, -PHI},     {ZERO, -INV_PHI, -PHI}
    },                                                      //  15
    {
        {ZERO, INV_PHI, -PHI},     {-ONE, ONE, -ONE}
    },                                                      //  16
    {
        {ZERO, INV_PHI, -PHI},     {ONE, ONE, -ONE}
    },                                                      //  17
    {
        {-ONE, -ONE, -ONE},        {ZERO, -INV_PHI, -PHI}
    },                                                      //  18
    {
        {ONE, -ONE, -ONE},         {ZERO, -INV_PHI, -PHI}
    },                                                      //  19
    {
        {PHI, ZERO, INV_PHI},         {PHI, ZERO, -INV_PHI}
    },                                                      //  20
    {
        {PHI, ZERO, INV_PHI},         {ONE, ONE, ONE}
    },                                                      //  21
    {
        {PHI, ZERO, INV_PHI},         {ONE, -ONE, ONE}
    },                                                      //  22
    {
        {ONE, ONE, -ONE},             {PHI, ZERO, -INV_PHI}
    },                                                      //  23
    {
        {ONE, -ONE, -ONE},            {PHI, ZERO, -INV_PHI}
    },                                                      //  24
    {
        {-PHI, ZERO, INV_PHI},     {-PHI, ZERO, -INV_PHI}
    },                                                      //  25
    {
        {-PHI, ZERO, INV_PHI},     {-ONE, ONE, ONE}
    },                                                      //  26
    {
        {-PHI, ZERO, INV_PHI},     {-ONE, -ONE, ONE}
    },                                                      //  27
    {
        {-ONE, ONE, -ONE},         {-PHI, ZERO, -INV_PHI}
    },                                                      //  28
    {
        {-ONE, -ONE, -ONE},        {-PHI, ZERO, -INV_PHI}
    },                                                      //  29
};

/*
 * Hypercube _q coordinate points
 */
const qVector4D gc_qHypercube4D[SHAPE_HYPERCUBE_SIZE][2] = {
    // w=1 cube
    {
        {-_Q(ONE), -_Q(ONE), -_Q(ONE),
         _Q(ONE)},   { _Q(ONE), -_Q(ONE), -_Q(ONE), _Q(ONE)}
    },
    {
        {-_Q(ONE), -_Q(ONE), -_Q(ONE),
         _Q(ONE)},   {-_Q(ONE),  _Q(ONE), -_Q(ONE), _Q(ONE)}
    },
    {
        {-_Q(ONE),  _Q(ONE), -_Q(ONE),
         _Q(ONE)},   { _Q(ONE),  _Q(ONE), -_Q(ONE), _Q(ONE)}
    },
    {
        { _Q(ONE), -_Q(ONE), -_Q(ONE),
          _Q(ONE)},   { _Q(ONE),  _Q(ONE), -_Q(ONE), _Q(ONE)}
    },
    {
        {-_Q(ONE), -_Q(ONE),  _Q(ONE),
         _Q(ONE)},   { _Q(ONE), -_Q(ONE),  _Q(ONE), _Q(ONE)}
    },
    {
        {-_Q(ONE), -_Q(ONE),  _Q(ONE),
         _Q(ONE)},   {-_Q(ONE),  _Q(ONE),  _Q(ONE), _Q(ONE)}
    },
    {
        {-_Q(ONE),  _Q(ONE),  _Q(ONE),
         _Q(ONE)},   { _Q(ONE),  _Q(ONE),  _Q(ONE), _Q(ONE)}
    },
    {
        { _Q(ONE), -_Q(ONE),  _Q(ONE),
          _Q(ONE)},   { _Q(ONE),  _Q(ONE),  _Q(ONE), _Q(ONE)}
    },
    {
        {-_Q(ONE), -_Q(ONE),  _Q(ONE),
         _Q(ONE)},   {-_Q(ONE), -_Q(ONE), -_Q(ONE), _Q(ONE)}
    },
    {
        {-_Q(ONE),  _Q(ONE),  _Q(ONE),
         _Q(ONE)},   {-_Q(ONE),  _Q(ONE), -_Q(ONE), _Q(ONE)}
    },
    {
        { _Q(ONE),  _Q(ONE),  _Q(ONE),
          _Q(ONE)},   { _Q(ONE),  _Q(ONE), -_Q(ONE), _Q(ONE)}
    },
    {
        { _Q(ONE), -_Q(ONE),  _Q(ONE),
          _Q(ONE)},   { _Q(ONE), -_Q(ONE), -_Q(ONE), _Q(ONE)}
    },
    // w=-1 cube
    {
        {-_Q(ONE), -_Q(ONE), -_Q(ONE),
         -_Q(ONE)},  { _Q(ONE), -_Q(ONE), -_Q(ONE), -_Q(ONE)}
    },
    {
        {-_Q(ONE), -_Q(ONE), -_Q(ONE),
         -_Q(ONE)},  {-_Q(ONE),  _Q(ONE), -_Q(ONE), -_Q(ONE)}
    },
    {
        {-_Q(ONE),  _Q(ONE), -_Q(ONE),
         -_Q(ONE)},  { _Q(ONE),  _Q(ONE), -_Q(ONE), -_Q(ONE)}
    },
    {
        { _Q(ONE), -_Q(ONE), -_Q(ONE),
          -_Q(ONE)},  { _Q(ONE),  _Q(ONE), -_Q(ONE), -_Q(ONE)}
    },
    {
        {-_Q(ONE), -_Q(ONE),  _Q(ONE),
         -_Q(ONE)},  { _Q(ONE), -_Q(ONE),  _Q(ONE), -_Q(ONE)}
    },
    {
        {-_Q(ONE), -_Q(ONE),  _Q(ONE),
         -_Q(ONE)},  {-_Q(ONE),  _Q(ONE),  _Q(ONE), -_Q(ONE)}
    },
    {
        {-_Q(ONE),  _Q(ONE),  _Q(ONE),
         -_Q(ONE)},  { _Q(ONE),  _Q(ONE),  _Q(ONE), -_Q(ONE)}
    },
    {
        { _Q(ONE), -_Q(ONE),  _Q(ONE),
          -_Q(ONE)},  { _Q(ONE),  _Q(ONE),  _Q(ONE), -_Q(ONE)}
    },
    {
        {-_Q(ONE), -_Q(ONE),  _Q(ONE),
         -_Q(ONE)},  {-_Q(ONE), -_Q(ONE), -_Q(ONE), -_Q(ONE)}
    },
    {
        {-_Q(ONE),  _Q(ONE),  _Q(ONE),
         -_Q(ONE)},  {-_Q(ONE),  _Q(ONE), -_Q(ONE), -_Q(ONE)}
    },
    {
        { _Q(ONE),  _Q(ONE),  _Q(ONE),
          -_Q(ONE)},  { _Q(ONE),  _Q(ONE), -_Q(ONE), -_Q(ONE)}
    },
    {
        { _Q(ONE), -_Q(ONE),  _Q(ONE),
          -_Q(ONE)},  { _Q(ONE), -_Q(ONE), -_Q(ONE), -_Q(ONE)}
    },
    // cube connections
    {
        { _Q(ONE),  _Q(ONE),  _Q(ONE),
          _Q(ONE)},   { _Q(ONE),  _Q(ONE),  _Q(ONE), -_Q(ONE)}
    },
    {
        { _Q(ONE),  _Q(ONE), -_Q(ONE),
          _Q(ONE)},   { _Q(ONE),  _Q(ONE), -_Q(ONE), -_Q(ONE)}
    },
    {
        { _Q(ONE), -_Q(ONE),  _Q(ONE),
          _Q(ONE)},   { _Q(ONE), -_Q(ONE),  _Q(ONE), -_Q(ONE)}
    },
    {
        { _Q(ONE), -_Q(ONE), -_Q(ONE),
          _Q(ONE)},   { _Q(ONE), -_Q(ONE), -_Q(ONE), -_Q(ONE)}
    },
    {
        {-_Q(ONE),  _Q(ONE),  _Q(ONE),
         _Q(ONE)},   {-_Q(ONE),  _Q(ONE),  _Q(ONE), -_Q(ONE)}
    },
    {
        {-_Q(ONE),  _Q(ONE), -_Q(ONE),
         _Q(ONE)},   {-_Q(ONE),  _Q(ONE), -_Q(ONE), -_Q(ONE)}
    },
    {
        {-_Q(ONE), -_Q(ONE),  _Q(ONE),
         _Q(ONE)},   {-_Q(ONE), -_Q(ONE),  _Q(ONE), -_Q(ONE)}
    },
    {
        {-_Q(ONE), -_Q(ONE), -_Q(ONE),
         _Q(ONE)},   {-_Q(ONE), -_Q(ONE), -_Q(ONE), -_Q(ONE)}
    }
};

/*
 * Hypercube float coordinate points
 */
const fVector4D gc_fHypercube4D[SHAPE_HYPERCUBE_SIZE][2] = {
    // w=1 cube
    {
        {-ONE, -ONE, -ONE, ONE},   { ONE, -ONE, -ONE, ONE}
    },
    {
        {-ONE, -ONE, -ONE, ONE},   {-ONE,  ONE, -ONE, ONE}
    },
    {
        {-ONE,  ONE, -ONE, ONE},   { ONE,  ONE, -ONE, ONE}
    },
    {
        { ONE, -ONE, -ONE, ONE},   { ONE,  ONE, -ONE, ONE}
    },
    {
        {-ONE, -ONE,  ONE, ONE},   { ONE, -ONE,  ONE, ONE}
    },
    {
        {-ONE, -ONE,  ONE, ONE},   {-ONE,  ONE,  ONE, ONE}
    },
    {
        {-ONE,  ONE,  ONE, ONE},   { ONE,  ONE,  ONE, ONE}
    },
    {
        { ONE, -ONE,  ONE, ONE},   { ONE,  ONE,  ONE, ONE}
    },
    {
        {-ONE, -ONE,  ONE, ONE},   {-ONE, -ONE, -ONE, ONE}
    },
    {
        {-ONE,  ONE,  ONE, ONE},   {-ONE,  ONE, -ONE, ONE}
    },
    {
        { ONE,  ONE,  ONE, ONE},   { ONE,  ONE, -ONE, ONE}
    },
    {
        { ONE, -ONE,  ONE, ONE},   { ONE, -ONE, -ONE, ONE}
    },
    // w=-1 cube
    {
        {-ONE, -ONE, -ONE, -ONE},  { ONE, -ONE, -ONE, -ONE}
    },
    {
        {-ONE, -ONE, -ONE, -ONE},  {-ONE,  ONE, -ONE, -ONE}
    },
    {
        {-ONE,  ONE, -ONE, -ONE},  { ONE,  ONE, -ONE, -ONE}
    },
    {
        { ONE, -ONE, -ONE, -ONE},  { ONE,  ONE, -ONE, -ONE}
    },
    {
        {-ONE, -ONE,  ONE, -ONE},  { ONE, -ONE,  ONE, -ONE}
    },
    {
        {-ONE, -ONE,  ONE, -ONE},  {-ONE,  ONE,  ONE, -ONE}
    },
    {
        {-ONE,  ONE,  ONE, -ONE},  { ONE,  ONE,  ONE, -ONE}
    },
    {
        { ONE, -ONE,  ONE, -ONE},  { ONE,  ONE,  ONE, -ONE}
    },
    {
        {-ONE, -ONE,  ONE, -ONE},  {-ONE, -ONE, -ONE, -ONE}
    },
    {
        {-ONE,  ONE,  ONE, -ONE},  {-ONE,  ONE, -ONE, -ONE}
    },
    {
        { ONE,  ONE,  ONE, -ONE},  { ONE,  ONE, -ONE, -ONE}
    },
    {
        { ONE, -ONE,  ONE, -ONE},  { ONE, -ONE, -ONE, -ONE}
    },
    // cube connections
    {
        { ONE,  ONE,  ONE, ONE},   { ONE,  ONE,  ONE, -ONE}
    },
    {
        { ONE,  ONE, -ONE, ONE},   { ONE,  ONE, -ONE, -ONE}
    },
    {
        { ONE, -ONE,  ONE, ONE},   { ONE, -ONE,  ONE, -ONE}
    },
    {
        { ONE, -ONE, -ONE, ONE},   { ONE, -ONE, -ONE, -ONE}
    },
    {
        {-ONE,  ONE,  ONE, ONE},   {-ONE,  ONE,  ONE, -ONE}
    },
    {
        {-ONE,  ONE, -ONE, ONE},   {-ONE,  ONE, -ONE, -ONE}
    },
    {
        {-ONE, -ONE,  ONE, ONE},   {-ONE, -ONE,  ONE, -ONE}
    },
    {
        {-ONE, -ONE, -ONE, ONE},   {-ONE, -ONE, -ONE, -ONE}
    }
};
